Import5 <-
function (x, tsCol=1, valCols, sumCols, verbose=0, seconds=120, maxGap=15, tz='GMT') {
  # x is data.frame, 
  # tsCol is column with datetime, 
  # valCols is vector with columns containing value data,
  # sumCols is vector with columns containing data that is summable,
  # verbose is integer with 0=no extra info in infoMsg, values 1 and 2 increase the information content
  # seconds is the time in seconds in a data row, 
  # max gap is the max number of contiguous rows that can be missing in the data

  errorExit = FALSE
  print(tsCol)
  print(x[1,])
  print(length(x[,tsCol]))
  print(x[length(x[,tsCol]),tsCol])   ####
  print ("-")
  print(x[1,tsCol])
  print ("+")
  print(seconds)
  #print(x[1,tsCol]+seconds)
  #print(x[,tsCol])
  print((x[length(x[,tsCol]),tsCol]) - x[1,tsCol]+seconds)
  timeLength <- as.numeric(x[length(x[,tsCol]),tsCol] - x[1,tsCol] + seconds)
  # create timeseries in seconds

  myTs <- ts(1:(as.numeric(x[length(x[,tsCol]),tsCol])-as.numeric(x[1,tsCol])+seconds), start=as.numeric(x[1,tsCol]), frequency=1)

  dataCol <- list()
  for (s in 1:ncol(x)){
    dataCol[[s]] <- vector("numeric", length(myTs))
    dataCol[[s]][c(1):c(length(myTs))] <- c(NA)
  }
  
  myTsData = as.data.frame(array(data=NA, dim=c(length(myTs),ncol(x)), dimnames = NULL))
  # this line bad?  vector too big 191.8 MB
  GapMsg <- ""
  MaxGapCnt <- 0
  totalGaps <- 0
  lastTs <- -seconds

  len <- length(x[,tsCol])
  for (s in 1:len){
    if ((verbose == 2) && (s %% 100 == 0))
      { print(paste("Processing row ",s))}
      tsInd = as.integer(as.numeric(x[s,tsCol])-as.numeric(x[1,tsCol]))
      curRow = x[s,]
      curRowData <- as.vector(curRow[sumCols])
      secInc = curRowData/seconds
      secSum <- secInc
      aSecInc <- array(data=0.0, dim=c(seconds,length(sumCols)), dimnames = NULL)
      dataCol[[tsCol]][c((tsInd+1):(tsInd+seconds))] <- c((tsInd+1):(tsInd+seconds))
      for (ind in valCols){
        dataCol[[ind]][c((tsInd+1):(tsInd+seconds))] <- curRow[ind]
      }
    for (ind in 1:length(sumCols)){
      sc <- sumCols[ind]
      dataCol[[sc]][c((tsInd+1):(tsInd+seconds))] <- as.numeric(secInc[ind])
    }
    
    if (tsInd != lastTs+seconds){
      # do missing row accounting
      totalGaps <- totalGaps + 1
      if (verbose == 2)
        { print(paste("Gap before row ",s))}
        CurGap <- (tsInd - lastTs - seconds) / seconds # in row count
        if (CurGap > MaxGapCnt){
          MaxGapCnt <- ceiling(CurGap)
        }
      # do interpolation of missing data rows

      missingValCnt <- tsInd - lastTs - seconds
      for (ind in c(valCols, sumCols)){
        stepInc <- (as.numeric(dataCol[[ind]][c(tsInd+1)]) - as.numeric(dataCol[[ind]][c(lastTs+seconds)])) / (missingValCnt + 1)
        for (val in c(1:missingValCnt)){
          dataCol[[ind]][lastTs+seconds+val] <- as.numeric(dataCol[[ind]][lastTs+seconds]) + val*stepInc
        }
      }
    }
    lastTs <- tsInd
  }
  
  percentMissing <- as.numeric(totalGaps / as.numeric(length(x[,tsCol]))) / 100.0
  
  if (MaxGapCnt >= maxGap){
    print(paste("Data has too large of a gap to be usable. Number of contiguous missing rows: ", MaxGapCnt))
    return(NULL)
    errorExit = TRUE
  } else
  {
    if (MaxGapCnt == 0){
      print("Data is contiguous. No imputation performed.")
    }
  }
  
  if (errorExit == FALSE)
  {
  # rebin data to that given in seconds parm
  if (verbose == 2){
    print("Rebinning data.")}
    newRowCnt <- as.integer(length(myTs)/seconds)
    rebinData <- matrix(data = NA, nrow=newRowCnt, ncol=ncol(x), byrow=FALSE, dimnames=NULL)
    Indexes <- factor(rep(1:(newRowCnt*seconds), each=seconds, length=newRowCnt*seconds))
    for (s in valCols){
      rebinData[,s] <- by(unlist(dataCol[[s]])[c(1:(newRowCnt*seconds))], Indexes, mean)
    }
    for (s in sumCols){
      rebinData[,s] <- by(unlist(dataCol[[s]])[c(1:(newRowCnt*seconds))], Indexes, sum)
  }
  dfOut <- as.data.frame(rebinData)
    
  names(dfOut) <- names(x)
  myDT <- c(1:nrow(rebinData))
  for (ind in 1:nrow(rebinData)){
    myDT[c(ind)] <- as.POSIXct(x[1,tsCol]+(ind-1)*seconds,tz=tz)
  }

  # apply timezone hack to get correct time
  origin <- as.POSIXct('1970-01-01 00:00:00',tz=tz) 
  offset <- as.numeric(origin)
  dfOut[,tsCol] <- as.POSIXct(myDT-offset, origin=origin,tz=tz)
  #dfOut[,tsCol] <- as.POSIXct(myDT, origin=origin,tz=tz)

  return(list(df=dfOut, maxGapCnt=MaxGapCnt, percentMissing=percentMissing))
 }
}
